/* Copyright (C) 2008 Victor Carceler <vcarceler@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  ChangeLog:
 *  2008-02-10 First version, based on L. Donnie Smith's nunchuk_acc
 *  * Initial ChangeLog
 *  * made global variables static
 */


#include "wmplugin.h"



static int rest_threshold = 5;		// Minimun actuation from rest
static int rest = 127;			// Rest value
static int input_cutoff = 30;		// Limit from the ends (0 and 255)
static int scale = 15;			// Scale factor
static int min_threshold;
static int max_threshold;
static int max_input;
static int min_input;
static float m;
static float n;

static unsigned char info_init = 0;
static struct wmplugin_info info;
static struct wmplugin_data data;

static cwiid_wiimote_t *wiimote;

static struct acc_cal acc_cal;
static int plugin_id;

wmplugin_info_t wmplugin_info;
wmplugin_init_t wmplugin_init;
wmplugin_exec_t wmplugin_exec;
static void process_nunchuk(struct cwiid_nunchuk_mesg *mesg);


struct wmplugin_info *wmplugin_info() {
	if (!info_init) {
		info.button_count = 0;
		info.axis_count = 2;
		info.axis_info[0].name = "X";
		info.axis_info[0].type = WMPLUGIN_ABS | WMPLUGIN_REL;
		info.axis_info[0].max  = scale;
		info.axis_info[0].min  = scale * -1;
		info.axis_info[0].fuzz = 0;
		info.axis_info[0].flat = 0;
		info.axis_info[1].name = "Y";
		info.axis_info[1].type = WMPLUGIN_ABS | WMPLUGIN_REL;
		info.axis_info[1].max  = scale;
		info.axis_info[1].min  = scale * -1;
		info.axis_info[1].fuzz = 0;
		info.axis_info[1].flat = 0;
		info.param_count = 2;
		info.param_info[0].name = "Rest_Threshold";
		info.param_info[0].type = WMPLUGIN_PARAM_INT;
		info.param_info[0].ptr = &rest_threshold;
		info.param_info[1].name = "Scale";
		info.param_info[1].type = WMPLUGIN_PARAM_INT;
		info.param_info[1].ptr = &scale;
		info_init = 1;
	}
	return &info;
}

int wmplugin_init(int id, cwiid_wiimote_t *arg_wiimote)
{
	plugin_id = id;
	wiimote = arg_wiimote;
	data.buttons = 0;
	data.axes[0].valid = 1;
	data.axes[1].valid = 1;
	if (wmplugin_set_rpt_mode(id, CWIID_RPT_STATUS | CWIID_RPT_NUNCHUK)) {
		return -1;
	}

	max_input = 255 - input_cutoff;
	min_input = 0 + input_cutoff;
	m = 2.0 / ((float)(max_input - min_input));
	n = ((float)((-1*max_input) - min_input)) / ((float)(max_input - min_input));
	min_threshold = rest - rest_threshold;
	max_threshold = rest + rest_threshold;

	return 0;
}

struct wmplugin_data *wmplugin_exec(int mesg_count, union cwiid_mesg mesg[])
{
	int i;
	enum cwiid_ext_type ext_type = CWIID_EXT_NONE;
	struct wmplugin_data *ret = NULL;

	for (i=0; i < mesg_count; i++) {
		switch (mesg[i].type) {
		case CWIID_MESG_STATUS:
			if ((mesg[i].status_mesg.ext_type == CWIID_EXT_NUNCHUK) &&
			  (ext_type != CWIID_EXT_NUNCHUK)) {
				if (cwiid_get_acc_cal(wiimote, CWIID_EXT_NUNCHUK, &acc_cal)) {
					wmplugin_err(plugin_id, "calibration error");
				}
			}
			ext_type = mesg[i].status_mesg.ext_type;
			break;
		case CWIID_MESG_NUNCHUK:
			process_nunchuk(&mesg[i].nunchuk_mesg);
			ret = &data;
			break;
		default:
			break;
		}
	}

	return ret;
}

static void process_nunchuk(struct cwiid_nunchuk_mesg *mesg)
{
	int x,y;
	x = (int)mesg->stick[0];
	y = (int)mesg->stick[1];

	if ( (x < min_threshold) || (x > max_threshold) ) {
		data.axes[0].valid = 1;
		data.axes[0].value = (int)( (m*x+n) * scale );
	}
	else {
		data.axes[0].valid = 0;
	}

	if ( (y < min_threshold) || (y > max_threshold) ) {
		data.axes[1].valid = 1;
		data.axes[1].value = (int)( (m*y+n) * scale );
	}
	else {
		data.axes[1].valid = 0;
	}
}
